#include <windows.h>
#include <tchar.h>
#include <memory.h>

#include "resource.h"
#include "ppongo.h"

BOOL	OpenPort(void)
{
	// Try to open the serial port.
    if(INVALID_HANDLE_VALUE == (hPort = CreateFile(
									v_szDevName,
									GENERIC_READ | GENERIC_WRITE,
									0, 
									NULL, 
									OPEN_EXISTING,
									0,
									NULL
									)))
	{
		MessageBox (NULL, TEXT("Unable to open port"), TEXT("Error"), MB_OK);
		return FALSE;
    }

	return TRUE;
}

BOOL	InitPort (void)
{
	DWORD	ThreadID;
	DCB		PortDCB;
    COMMTIMEOUTS CommTimeouts;

	if(!OpenPort())
		return FALSE;

	// Initialize the port info.
    PortDCB.DCBlength = sizeof(DCB);

	// Get the port info.
    GetCommState(hPort, &PortDCB);

	PortDCB.BaudRate		= 9600;
	PortDCB.fBinary			= TRUE;
	PortDCB.fParity			= TRUE;
	PortDCB.fOutxCtsFlow    = FALSE;  // ignore possible hangups
	PortDCB.fOutxDsrFlow    = FALSE;    // don't wait on the DSR line
	PortDCB.fDtrControl     = DTR_CONTROL_ENABLE;
	PortDCB.fDsrSensitivity = FALSE;
	PortDCB.fTXContinueOnXoff = TRUE;
	PortDCB.fOutX           = FALSE; // no XON/XOFF control
	PortDCB.fInX            = FALSE;
	PortDCB.fErrorChar      = FALSE;
	PortDCB.fNull           = FALSE;
	PortDCB.fRtsControl     = RTS_CONTROL_ENABLE;
	PortDCB.fAbortOnError   = FALSE;
	PortDCB.ByteSize        = 8;
	PortDCB.Parity          = NOPARITY;
	PortDCB.StopBits        = ONESTOPBIT;

	SetCommState(hPort, &PortDCB);
	
	PrintDCB (hPort);

	// Set comm timeouts (nonblocking read, and write waiting for completion).
	GetCommTimeouts(hPort, &CommTimeouts);
	CommTimeouts.ReadIntervalTimeout = MAXDWORD;
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;
	CommTimeouts.ReadTotalTimeoutConstant = 0;	
	CommTimeouts.WriteTotalTimeoutMultiplier = 0;
	CommTimeouts.WriteTotalTimeoutConstant = 0;	
	SetCommTimeouts(hPort, &CommTimeouts);
	
	// Create the read thread
	CloseHandle (CreateThread (NULL, 0, ReadThread, 0, 0, &ThreadID));

	EscapeCommFunction (hPort, SETDTR);
	EscapeCommFunction (hPort, SETRTS);

	return TRUE;
}

BOOL	ClosePort (void)
{
    if ( hPort != INVALID_HANDLE_VALUE ) 
	{
		CloseHandle(hPort);
		hPort = INVALID_HANDLE_VALUE;
		return TRUE;
	}
	return FALSE;
}

void PortWrite (BYTE	Byte)
{
	DWORD		Len;
#ifdef _WIN32_WCE
	WriteFile (hPort, &Byte, 1, &Len, 0);
#else
	OVERLAPPED	OverLapped;

	memset ((char *)&OverLapped, 0, sizeof(OVERLAPPED));
	OverLapped.hEvent = CreateEvent (NULL, TRUE, TRUE, 0);

	if (!WriteFile (hPort, &Byte, 1, &Len, &OverLapped)) 
	{
		if (ERROR_IO_PENDING == GetLastError()) 
		{
			WaitForSingleObject (OverLapped.hEvent, INFINITE);
		}
	}
	CloseHandle (OverLapped.hEvent);
#endif
}

DWORD WINAPI ReadThread (LPVOID pvarg)
{
	BYTE	InByte;
	DWORD	BytesTransferred;
	DWORD	fdwCommMask;

#ifndef _WIN32_WCE
	OVERLAPPED	OverLapped;

	memset ((char *)&OverLapped, 0, sizeof(OVERLAPPED));
	OverLapped.hEvent = CreateEvent (NULL, TRUE, TRUE, 0);
#endif


	SetCommMask (hPort, EV_RXCHAR | EV_CTS | EV_DSR | EV_RLSD | EV_RING);

	while (!v_bStopThread) 
	{
#ifdef _WIN32_WCE
		WaitCommEvent (hPort, &fdwCommMask, 0);
#else
		if (!WaitCommEvent (hPort, &fdwCommMask, &OverLapped)) 
		{
			if (GetLastError() == ERROR_IO_PENDING) 
			{
				WaitForSingleObject (OverLapped.hEvent, INFINITE);
			}
        }
#endif

		// Reset the comm Mask
		SetCommMask (hPort, EV_RXCHAR | EV_CTS | EV_DSR | EV_RING);

		if (fdwCommMask & EV_RXCHAR) 
		{
			// Loop waiting for data.
			do 
			{
#ifdef _WIN32_WCE
				ReadFile (hPort, &InByte, 1, &BytesTransferred, 0);
#else
				if (!ReadFile (hPort, &InByte, 1, &BytesTransferred,
							   &OverLapped)) 
				{
					if (ERROR_IO_PENDING == GetLastError()) 
					{
						WaitForSingleObject (OverLapped.hEvent, INFINITE);
					}
                }
#endif
				if (1 == BytesTransferred) 
				{
					ProcessChar (InByte);
				}
			} 
			while (1 == BytesTransferred);
		}

		// Should we check this anyway????
		GetCommModemStatus (hPort, &fdwCommMask);
	}

#ifndef _WIN32_WCE
	CloseHandle (OverLapped.hEvent);
#endif
	return 0;
}

void PrintDCB(HANDLE hPort)
{
	DCB	PortDCB;

	PortDCB.DCBlength = sizeof(DCB);
	GetCommState (hPort, &PortDCB);

	DEBUGMSG(1, (TEXT("\r\nDCBLength=%d BaudRate=%d fBinary=%s fParity=%s\r\n"),
		PortDCB.DCBlength, PortDCB.BaudRate,
		PortDCB.fBinary ? TEXT("True") : TEXT("False"),
		PortDCB.fParity ? TEXT("True") : TEXT("False")));
	DEBUGMSG(1, (TEXT("fOutxCtsFlow=%s fOutxDsrFlow=%s fDtrControl=%d\r\n"),
		PortDCB.fOutxCtsFlow ? TEXT("True") : TEXT("False"),
		PortDCB.fOutxDsrFlow ? TEXT("True") : TEXT("False"),
		PortDCB.fDtrControl));
	DEBUGMSG(1, (TEXT("fDsrSensitivity=%s fTXContinueOnXoff=%s fOutX=%s fInX=%s\r\n"),
		PortDCB.fDsrSensitivity ? TEXT("True") : TEXT("False"),
		PortDCB.fTXContinueOnXoff ? TEXT("True") : TEXT("False"),
		PortDCB.fOutX ? TEXT("True") : TEXT("False"),
		PortDCB.fInX ? TEXT("True") : TEXT("False")));
	DEBUGMSG(1, (TEXT("fErrorChar=%s fNull=%s fRtsControl=%d fAbortOnError=%s\r\n"),
		PortDCB.fErrorChar ? TEXT("True") : TEXT("False"),
		PortDCB.fNull ? TEXT("True") : TEXT("False"),
		PortDCB.fRtsControl,
		PortDCB.fAbortOnError ? TEXT("True") : TEXT("False")));
	DEBUGMSG(1, (TEXT("XonLim=%d XoffLim=%d ByteSize=%d Parity=%d StopBits=%d\r\n"),
		PortDCB.XonLim, PortDCB.XoffLim, PortDCB.ByteSize,
		PortDCB.Parity, PortDCB.StopBits));
	DEBUGMSG(1, (TEXT("ErrorChar=0x%x EofChar=0x%x EvtChar=0x%x\r\n"),
		PortDCB.ErrorChar, PortDCB.EofChar, PortDCB.EvtChar));
}


